<?php

namespace HaoZiTeam\WPChatGPT\Service;

defined( 'ABSPATH' ) || exit;

use Exception;
use HaoZiTeam\WPChatGPT\Inc\EpayCore;
use HaoZiTeam\WPChatGPT\Inc\XunHuPay;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;
use Yansongda\Pay\Exceptions\InvalidArgumentException;
use Yansongda\Pay\Exceptions\InvalidConfigException;
use Yansongda\Pay\Exceptions\InvalidSignException;
use Yansongda\Pay\Pay;

/**
 * Class Bug
 * 实现插件的购买接口
 * @package HaoZiTeam\ChatGPT\Service
 */
class Buy {

	private $settings;
	private $no_login_notice = '请先登录';
	protected array $alipay;
	protected array $wxpay;

	public function __construct() {
		$this->settings = get_option( 'wp-chatgpt' );
		// 判断是否开启了免登录会话
		if ( $this->settings['custom-conversation'] ?? false ) {
			if ( ! is_user_logged_in() && ! empty( $this->settings['custom-conversation-user'] ) ) {
				// 判断用户id是否存在
				$user = get_user_by( 'id', $this->settings['custom-conversation-user'] );
				if ( $user ) {
					wp_set_current_user( $user->ID, $user->user_login );
				}
			}
		}
		// 设置未登录提示
		if ( ! empty( $this->settings['no-login-notice'] ) ) {
			$this->no_login_notice = $this->settings['no-login-notice'];
		}

		register_rest_route( 'wp-chatgpt', 'package', array(
			'methods'             => WP_REST_Server::READABLE,
			'callback'            => array( $this, 'package' ),
			'permission_callback' => '__return_true',
		) );

		register_rest_route( 'wp-chatgpt', 'pay-type', array(
			'methods'             => WP_REST_Server::READABLE,
			'callback'            => array( $this, 'pay_type' ),
			'permission_callback' => '__return_true',
		) );

		register_rest_route( 'wp-chatgpt', 'create-order', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'create_order' ),
			'permission_callback' => '__return_true',
		) );

		register_rest_route( 'wp-chatgpt', 'check-order', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'check_order' ),
			'permission_callback' => '__return_true',
		) );

		register_rest_route( 'wp-chatgpt', 'epay-notify', array(
			'methods'             => WP_REST_Server::READABLE,
			'callback'            => array( $this, 'epay_notify' ),
			'permission_callback' => '__return_true',
		) );

		register_rest_route( 'wp-chatgpt', 'xh-notify', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'xh_notify' ),
			'permission_callback' => '__return_true',
		) );

		register_rest_route( 'wp-chatgpt', 'alipay-notify', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'alipay_notify' ),
			'permission_callback' => '__return_true',
		) );

		register_rest_route( 'wp-chatgpt', 'wxpay-notify', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'wxpay_notify' ),
			'permission_callback' => '__return_true',
		) );

		register_rest_route( 'wp-chatgpt', 'order-history', array(
			'methods'             => WP_REST_Server::READABLE,
			'callback'            => array( $this, 'order_history' ),
			'permission_callback' => '__return_true',
		) );
	}

	/**
	 * 获取套餐信息
	 */
	public function package(): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		$packages = $this->settings['pay-packages'] ?? [];
		$data     = [];

		if ( empty( $packages ) ) {
			return $this->error( '未设置套餐' );
		}

		foreach ( $packages as $key => $package ) {
			$data[] = [
				'package' => $key,
				'name'    => $package['package-name'],
				'price'   => $package['package-price'],
				'quota'   => $package['package-quota'],
			];
		}

		return $this->success( '获取套餐信息成功', $data );
	}

	/**
	 * 获取支付方式
	 */
	public function pay_type(): WP_REST_Response {
		$wxpay  = $this->settings['pay-wxpay-settings']['switch'] ?? false;
		$alipay = $this->settings['pay-alipay-settings']['switch'] ?? false;

		$data = [
			'wxpay'  => $wxpay,
			'alipay' => $alipay,
		];

		return $this->success( '获取支付方式成功', $data );
	}

	/**
	 * 创建订单
	 */
	public function create_order( WP_REST_Request $request ): WP_REST_Response {
		// 判断是否开启了免登录会话
		if ( $this->settings['custom-conversation'] ?? false ) {
			if ( get_current_user_id() == $this->settings['custom-conversation-user'] ?? 0 ) {
				return $this->error( $this->no_login_notice );
			}
		}
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 判断是否开启了付费
		if ( ! $this->settings['pay-switch'] ?? false ) {
			return $this->error( '未开启付费' );
		}

		// 获取套餐信息
		$packages = $this->settings['pay-packages'] ?? [];
		if ( ! isset( $packages[ $request->get_param( 'package' ) ] ) ) {
			return $this->error( '套餐不存在' );
		}

		// 判断支付方式
		if ( ! in_array( $request->get_param( 'pay_type' ), [ 'alipay', 'wxpay' ] ) ) {
			return $this->error( '支付方式错误' );
		}

		// 创建订单
		$order = [
			'id'       => uniqid(),
			'package'  => $request->get_param( 'package' ),
			'name'     => $packages[ $request->get_param( 'package' ) ]['package-name'],
			'price'    => $packages[ $request->get_param( 'package' ) ]['package-price'],
			'quota'    => $packages[ $request->get_param( 'package' ) ]['package-quota'],
			'pay_type' => $request->get_param( 'pay_type' ),
			'paid'     => 0,
			'created'  => time(),
		];

		// 初始化支付
		if ( $order['pay_type'] === 'wxpay' ) {
			$config = $this->settings['pay-wxpay-settings'] ?? [];
		} else {
			$config = $this->settings['pay-alipay-settings'] ?? [];
		}
		if ( ! $config['switch'] ?? false ) {
			return $this->error( '当前支付方式未开启' );
		}
		if ( empty( $config['type'] ) ) {
			return $this->error( '未正确配置支付参数' );
		}
		$type = $config['type'];
		if ( $type === 'official' ) {
			self::official_init( $config );
			if ( $order['pay_type'] === 'wxpay' ) {
				// 判断是扫码支付还是H5支付
				if ( $config['official-settings']['interface-type'] == 'wap' ) {
					try {
						$wechat = Pay::wechat( $this->wxpay )->wap( [
							'out_trade_no' => $order['id'],
							'body'         => '购买 ' . $order['name'] . ' 套餐',
							'total_fee'    => strval( $order['price'] * 100 ),
							'attach'       => get_current_user_id(),
						] );
					} catch ( Exception $e ) {
						return $this->error( $e->getMessage() );
					}
					$pay_url = $wechat->getTargetUrl();
				} else if ( $config['official-settings']['interface-type'] == 'scan' ) {
					try {
						$wechat = Pay::wechat( $this->wxpay )->scan( [
							'out_trade_no' => $order['id'],
							'body'         => '购买 ' . $order['name'] . ' 套餐',
							'total_fee'    => strval( $order['price'] * 100 ),
							'attach'       => get_current_user_id(),
						] );
					} catch ( Exception $e ) {
						return $this->error( $e->getMessage() );
					}
					$pay_url = home_url( '/wp-content/plugins/wp-chatgpt/frontend/qrcode.php?app=wxpay&url=' . urlencode( $wechat->code_url ) );
				} else if ( $config['official-settings']['interface-type'] == 'auto' ) {
					if ( wp_is_mobile() ) {
						try {
							$wechat = Pay::wechat( $this->wxpay )->wap( [
								'out_trade_no' => $order['id'],
								'body'         => '购买 ' . $order['name'] . ' 套餐',
								'total_fee'    => strval( $order['price'] * 100 ),
								'attach'       => get_current_user_id(),
							] );
						} catch ( Exception $e ) {
							return $this->error( $e->getMessage() );
						}
						$pay_url = $wechat->getTargetUrl();
					} else {
						try {
							$wechat = Pay::wechat( $this->wxpay )->scan( [
								'out_trade_no' => $order['id'],
								'body'         => '购买 ' . $order['name'] . ' 套餐',
								'total_fee'    => strval( $order['price'] * 100 ),
								'attach'       => get_current_user_id(),
							] );
						} catch ( Exception $e ) {
							return $this->error( $e->getMessage() );
						}
						$pay_url = home_url( '/wp-content/plugins/wp-chatgpt/frontend/qrcode.php?app=wxpay&url=' . urlencode( $wechat->code_url ) );
					}
				} else {
					return $this->error( '未正确配置微信支付接口类型' );
				}
			} else {
				// 扫码支付
				if ( $config['official-settings']['interface-type'] == 'scan' ) {
					try {
						$alipay = Pay::alipay( $this->alipay )->scan( [
							'out_trade_no' => $order['id'],
							'subject'      => '购买 ' . $order['name'] . ' 套餐',
							'total_amount' => $order['price'],
							'body'         => get_current_user_id(),
						] );
					} catch ( Exception $e ) {
						return $this->error( $e->getMessage() );
					}
					$pay_url = home_url( '/wp-content/plugins/wp-chatgpt/frontend/qrcode.php?app=alipay&url=' . urlencode( $alipay->qr_code ) );
				} else {
					return $this->error( '未正确配置支付宝支付接口类型' );
				}
			}
		} else if ( $type === 'epay' ) {
			$epay_apiurl = $config['epay-settings']['apiurl'] ?? '';
			$epay_pid    = $config['epay-settings']['pid'] ?? '';
			$epay_key    = $config['epay-settings']['key'] ?? '';
			if ( empty( $epay_apiurl ) || empty( $epay_pid ) || empty( $epay_key ) ) {
				return $this->error( '未正确配置易支付支付参数' );
			}

			$epay = new EpayCore( [
				'apiurl' => $epay_apiurl,
				'pid'    => $epay_pid,
				'key'    => $epay_key,
			] );

			$parameter = [
				'pid'          => $epay_pid,
				'type'         => $order['pay_type'],
				'notify_url'   => home_url( '/wp-json/wp-chatgpt/epay-notify' ) . '?user_id=' . get_current_user_id(),
				'return_url'   => empty( $this->settings['pay-return-url'] ) ? home_url() : $this->settings['pay-return-url'],
				'out_trade_no' => $order['id'],
				'name'         => '购买 ' . $order['name'] . ' 套餐',
				'money'        => $order['price'],
			];
			// 生成支付链接
			$pay_url = $epay->getPayLink( $parameter );
		} elseif ( $type === 'hupijiao' ) {
			$apiurl    = $config['hupijiao-settings']['apiurl'] ?? '';
			$appid     = $config['hupijiao-settings']['appid'] ?? '';
			$appsecret = $config['hupijiao-settings']['appsecret'] ?? '';
			if ( $order['pay_type'] == 'wxpay' ) {
				$order['pay_type'] = 'wechat';// 虎皮椒支付的微信支付类型为wechat
			}
			if ( empty( $apiurl ) || empty( $appid ) || empty( $appsecret ) ) {
				return $this->error( '未正确配置虎皮椒支付参数' );
			}

			$hupijiao = new XunHuPay( $appid, $appsecret, $apiurl );
			$data     = [
				'trade_order_id' => $order['id'],
				'title'          => '购买 ' . $order['name'] . ' 套餐',
				'total_fee'      => $order['price'],
				'notify_url'     => home_url( '/wp-json/wp-chatgpt/xh-notify' ) . '?user_id=' . get_current_user_id() . '&pay_type=' . $order['pay_type'],
				'return_url'     => empty( $this->settings['pay-return-url'] ) ? home_url() : $this->settings['pay-return-url'],
				'attach'         => get_current_user_id(),
				'payment'        => $order['pay_type'],
			];
			$pay      = $hupijiao->getPayUrl( $data );
			if ( $pay['status'] ) {
				$pay_url = $pay['url'];
			} else {
				return $this->error( $pay['message'] );
			}
		} else {
			return $this->error( '无可用的支付方式' );
		}

		// 保存订单到用户元数据
		$orders = get_user_meta( get_current_user_id(), 'wp-chatgpt-orders', true );
		if ( ! is_array( $orders ) ) {
			$orders = [];
		}
		$orders[] = $order;
		update_user_meta( get_current_user_id(), 'wp-chatgpt-orders', $orders );

		// 返回订单信息
		$order['pay_url'] = $pay_url;

		return $this->success( '订单创建成功', $order );
	}

	/**
	 * 检查订单
	 */
	public function check_order( WP_REST_Request $request ): WP_REST_Response {
		// 判断是否开启了免登录会话
		if ( $this->settings['custom-conversation'] ?? false ) {
			if ( get_current_user_id() == $this->settings['custom-conversation-user'] ?? 0 ) {
				return $this->error( $this->no_login_notice );
			}
		}
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 获取订单
		$orders = get_user_meta( get_current_user_id(), 'wp-chatgpt-orders', true );
		if ( ! is_array( $orders ) ) {
			$orders = [];
		}

		// 检查订单
		foreach ( $orders as $order ) {
			if ( $order['id'] === $request->get_param( 'id' ) ) {
				if ( $order['paid'] ) {
					return $this->success( '订单支付成功' );
				} else {
					return $this->error( '订单未支付' );
				}
			}
		}

		return $this->error( '订单不存在' );
	}

	/**
	 * 易支付回调
	 */
	public function epay_notify( WP_REST_Request $request ): WP_REST_Response {
		// 获取支付方式
		$pay_type = $request->get_param( 'type' ) ?? '';
		if ( empty( $pay_type ) ) {
			exit( 'fail' );
		}
		if ( $pay_type === 'wxpay' ) {
			$config = $this->settings['pay-wxpay-settings'] ?? [];
		} else {
			$config = $this->settings['pay-alipay-settings'] ?? [];
		}

		$apiurl = $config['epay-settings']['apiurl'] ?? '';
		$pid    = $config['epay-settings']['pid'] ?? '';
		$key    = $config['epay-settings']['key'] ?? '';
		if ( empty( $apiurl ) || empty( $pid ) || empty( $key ) ) {
			exit( 'fail' );
		}

		$epay          = new EpayCore( [
			'apiurl' => $apiurl,
			'pid'    => $pid,
			'key'    => $key,
		] );
		$verify_result = $epay->verifyNotify();

		if ( $verify_result ) {
			$user_id      = $request->get_param( 'user_id' ) ?? 0;
			$out_trade_no = $request->get_param( 'out_trade_no' );
			$trade_status = $request->get_param( 'trade_status' );

			if ( ! $user_id ) {
				return $this->error( '用户不存在' );
			}

			wp_set_current_user( $user_id );

			if ( $trade_status == 'TRADE_SUCCESS' ) {
				self::update_order( $out_trade_no );
				exit( 'success' );
			}
		}
		exit( 'fail' );
	}

	/**
	 * 虎皮椒回调
	 */
	public function xh_notify( WP_REST_Request $request ): WP_REST_Response {
		$pay_type = $request->get_param( 'pay_type' ) ?? '';
		if ( empty( $pay_type ) ) {
			exit( 'fail' );
		}
		if ( $pay_type === 'wechat' ) {
			$config = $this->settings['pay-wxpay-settings'] ?? [];
		} else {
			$config = $this->settings['pay-alipay-settings'] ?? [];
		}

		$apiurl    = $config['hupijiao-settings']['apiurl'] ?? '';
		$appid     = $config['hupijiao-settings']['appid'] ?? '';
		$appsecret = $config['hupijiao-settings']['appsecret'] ?? '';
		if ( empty( $apiurl ) || empty( $appid ) || empty( $appsecret ) ) {
			exit( 'fail' );
		}

		$hupijiao = new XunHuPay( $appid, $appsecret, $apiurl );
		$check    = $hupijiao->notifyCheck();

		if ( $check ) {
			$user_id        = $request->get_param( 'user_id' ) ?? 0;
			$trade_order_id = $request->get_param( 'trade_order_id' );
			$status         = $request->get_param( 'status' );

			if ( ! $user_id ) {
				return $this->error( '用户不存在' );
			}

			wp_set_current_user( $user_id );

			if ( $status == 'OD' ) {
				self::update_order( $trade_order_id );
				exit( 'success' );
			}
		}
		exit( 'fail' );
	}

	/**
	 * 支付宝回调
	 */
	public function alipay_notify(): WP_REST_Response {
		$config = $this->settings['pay-alipay-settings'] ?? [];
		self::official_init( $config );

		try {
			$result = Pay::alipay( $this->alipay )->verify();
		} catch ( InvalidConfigException|InvalidSignException $e ) {
			exit( 'fail' );
		}

		// 获取订单
		$user_id      = $result['body'] ?? '';
		$out_trade_no = $result['out_trade_no'] ?? '';
		$trade_status = $result['trade_status'] ?? '';
		if ( empty( $user_id ) || empty( $out_trade_no ) || empty( $trade_status ) ) {
			exit( 'fail' );
		}
		if ( $trade_status == 'TRADE_SUCCESS' ) {
			wp_set_current_user( $user_id );
			self::update_order( $out_trade_no );
			exit( 'success' );
		}
		exit( 'fail' );
	}

	/**
	 * 微信回调
	 */
	public function wxpay_notify(): WP_REST_Response {
		$config = $this->settings['pay-wxpay-settings'] ?? [];
		self::official_init( $config );

		try {
			$result = Pay::wechat( $this->wxpay )->verify();
		} catch ( InvalidArgumentException|InvalidSignException $e ) {
			header( 'HTTP/1.1 500 Internal Server Error' );
			exit( $e->getMessage() );
		}

		// 获取订单
		$user_id      = $result['attach'] ?? '';
		$out_trade_no = $result['out_trade_no'] ?? '';
		$result_code  = $result['result_code'] ?? '';
		if ( empty( $user_id ) || empty( $out_trade_no ) || empty( $result_code ) ) {
			exit( 'fail' );
		}
		if ( $result_code == 'SUCCESS' ) {
			wp_set_current_user( $user_id );
			self::update_order( $out_trade_no );
			$return = [
				'return_code' => 'SUCCESS',
				'return_msg'  => 'OK',
			];
			$xml    = self::toXml( $return );
			if ( $xml ) {
				header( 'Content-Type: application/xml' );
				exit( $xml );
			} else {
				header( 'HTTP/1.1 500 Internal Server Error' );
				exit( 'fail' );
			}
		}
		header( 'HTTP/1.1 500 Internal Server Error' );
		exit( 'fail' );
	}

	/**
	 * 订单记录
	 */
	public function order_history( WP_REST_Request $request ): WP_REST_Response {
		// 判断是否开启了免登录会话
		if ( $this->settings['custom-conversation'] ?? false ) {
			if ( get_current_user_id() == $this->settings['custom-conversation-user'] ?? 0 ) {
				return $this->error( $this->no_login_notice );
			}
		}
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		$orders = get_user_meta( get_current_user_id(), 'wp-chatgpt-orders', true );
		if ( ! is_array( $orders ) ) {
			$orders = [];
		}

		return $this->success( '获取成功', $orders );
	}

	/**
	 * 官方支付的初始化
	 */
	private function official_init( $config ): void {
		$this->alipay = [
			'app_id'         => $config['official-settings']['app-id'] ?? '',
			'notify_url'     => home_url( '/wp-json/wp-chatgpt/alipay-notify' ),
			'return_url'     => empty( $this->settings['pay-return-url'] ) ? home_url() : $this->settings['pay-return-url'],
			'ali_public_key' => str_replace( array(
				"\r\n",
				"\r",
				"\n"
			), "", $config['official-settings']['ali-public-key'] ?? '' ),
			// 加密方式： **RSA2**
			'private_key'    => str_replace( array(
				"\r\n",
				"\r",
				"\n"
			), "", $config['official-settings']['private-key'] ?? '' ),
			'log'            => [
				'file'  => './logs/alipay.log',
				'level' => 'info', // 建议生产环境等级调整为 info，开发环境为 debug
				'type'  => 'single', // optional, 可选 daily.
			],
			'http'           => [
				'timeout'         => 20,
				'connect_timeout' => 20,
			],
		];
		$this->wxpay  = [
			'app_id'     => $config['official-settings']['app-id'] ?? '',
			'mch_id'     => $config['official-settings']['mch-id'] ?? '',
			'key'        => $config['official-settings']['mch-secret-key'] ?? '',
			'notify_url' => home_url( '/wp-json/wp-chatgpt/wxpay-notify' ),
			'log'        => [
				'file'  => './logs/wechat.log',
				'level' => 'info', // 建议生产环境等级调整为 info，开发环境为 debug
				'type'  => 'single', // optional, 可选 daily.
			],
			'http'       => [
				'timeout'         => 20,
				'connect_timeout' => 20,
			],
		];
	}

	/**
	 * 更新订单记录
	 */
	private function update_order( $order_id ) {
		// 获取订单
		$orders = get_user_meta( get_current_user_id(), 'wp-chatgpt-orders', true );
		if ( ! is_array( $orders ) ) {
			$this->error( '订单不存在' );
		}
		foreach ( $orders as $k => $order ) {
			if ( $order['id'] === $order_id && $order['paid'] == 0 ) {
				$orders[ $k ]['paid'] = 1;
				// 更新用户配额
				$quota = get_user_meta( get_current_user_id(), 'wp-chatgpt-quota', true );
				if ( ! is_numeric( $quota ) ) {
					$quota = 0;
				}
				$quota += $order['quota'];
				update_user_meta( get_current_user_id(), 'wp-chatgpt-quota', $quota );
				break;
			}
		}
		update_user_meta( get_current_user_id(), 'wp-chatgpt-orders', $orders );
	}

	public static function toXml( $data ): string {
		if ( ! is_array( $data ) || count( $data ) <= 0 ) {
			return '';
		}

		$xml = '<xml>';
		foreach ( $data as $key => $val ) {
			$xml .= is_numeric( $val ) ? '<' . $key . '>' . $val . '</' . $key . '>' :
				'<' . $key . '><![CDATA[' . $val . ']]></' . $key . '>';
		}
		$xml .= '</xml>';

		return $xml;
	}

	public function success( $message = 'success', $data = null ): WP_REST_Response {
		return new WP_REST_Response( array( 'message' => $message, 'data' => $data, 'status' => 'Success' ) );
	}

	public function error( $message = 'error', $data = null ): WP_REST_Response {
		return new WP_REST_Response( array( 'message' => $message, 'data' => $data, 'status' => 'Failed' ) );
	}
}
