<?php

namespace HaoZiTeam\WPChatGPT\Service;

defined( 'ABSPATH' ) || exit;

use Exception;
use GuzzleHttp\Exception\GuzzleException;
use HaoZiTeam\ChatGPT\V1;
use HaoZiTeam\ChatGPT\V2;
use HaoZiTeam\WPChatGPT\Inc\YiYan;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

/**
 * Class Chat
 * 实现插件的对话接口
 * @package HaoZiTeam\ChatGPT\Service
 */
class Chat {

	private $settings;
	private $no_login_notice = '请先登录';

	public function __construct() {
		$this->settings = get_option( 'wp-chatgpt' );
		// 判断是否开启了免登录会话
		if ( $this->settings['custom-conversation'] ?? false ) {
			if ( ! is_user_logged_in() && ! empty( $this->settings['custom-conversation-user'] ) ) {
				// 判断用户id是否存在
				$user = get_user_by( 'id', $this->settings['custom-conversation-user'] );
				if ( $user ) {
					wp_set_current_user( $user->ID, $user->user_login );
				}
			}
		}
		// 设置未登录提示
		if ( ! empty( $this->settings['no-login-notice'] ) ) {
			$this->no_login_notice = $this->settings['no-login-notice'];
		}

		register_rest_route( 'wp-chatgpt', 'user-info', array(
			'methods'             => WP_REST_Server::READABLE,
			'callback'            => array( $this, 'user_info' ),
			'permission_callback' => '__return_true',
		) );
		register_rest_route( 'wp-chatgpt', 'message-process', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'message_process' ),
			'permission_callback' => '__return_true',
		) );
		register_rest_route( 'wp-chatgpt', 'delete-message', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'delete_message' ),
			'permission_callback' => '__return_true',
		) );
		register_rest_route( 'wp-chatgpt', 'conversation-list', array(
			'methods'             => WP_REST_Server::READABLE,
			'callback'            => array( $this, 'conversation_list' ),
			'permission_callback' => '__return_true',
		) );
		register_rest_route( 'wp-chatgpt', 'new-conversation', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'new_conversation' ),
			'permission_callback' => '__return_true',
		) );
		register_rest_route( 'wp-chatgpt', 'update-conversation', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'update_conversation' ),
			'permission_callback' => '__return_true',
		) );
		register_rest_route( 'wp-chatgpt', 'delete-conversation', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'delete_conversation' ),
			'permission_callback' => '__return_true',
		) );
		register_rest_route( 'wp-chatgpt', 'clear-conversation', array(
			'methods'             => WP_REST_Server::CREATABLE,
			'callback'            => array( $this, 'clear_conversation' ),
			'permission_callback' => '__return_true',
		) );
	}

	/**
	 * 获取用户信息
	 *
	 * @return WP_REST_Response
	 */
	public function user_info(): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		$user = wp_get_current_user();
		// 获取额度
		if ( ! empty( $this->settings['pay-switch'] ) && $this->settings['pay-switch'] && get_current_user_id() != $this->settings['custom-conversation-user'] ?? null ) {
			$quota = get_user_meta( $user->ID, 'wp-chatgpt-quota', true );
			if ( empty( $quota ) ) {
				$quota = 0;
			}
		} else {
			$quota = '无限';
		}

		$wxpay  = (bool) $this->settings['pay-wxpay-settings']['switch'] ?? false;
		$alipay = (bool) $this->settings['pay-alipay-settings']['switch'] ?? false;
		$pay    = false;
		if ( (bool) $this->settings['pay-switch'] ?? false ) {
			// 支付宝和微信需要开启一个，才能显示支付按钮
			if ( $wxpay || $alipay ) {
				// 未登录的用户不显示支付按钮
				if ( is_user_logged_in() ) {
					$pay = true;
				}
			}
		}

		return $this->success( '获取成功', [
			'name'        => $user->display_name,
			'avatar'      => get_avatar_url( $user->ID ),
			'description' => '剩余额度：' . $quota . ' 次',
			'gptAvatar'   => $this->settings['gpt-avatar']['url'] ?? '',
			'pay'         => $pay,
		] );
	}

	/**
	 * 聊天流式接口
	 * 通过缓冲区刷新的方式实现聊天
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_REST_Response
	 */
	public function message_process( WP_REST_Request $request ): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 检查授权
		$code       = "BcZPH02rbyHwefwCQFvI5bfPY577aEio";
		$auth_id    = $this->settings['auth-id'] ?? null;
		$auth_key   = $this->settings['auth-key'] ?? null;
		$time       = time();
		$auth_time  = get_transient( 'wp-chatgpt-auth-time' ) ?? '';
		$auth_cache = get_transient( 'wp-chatgpt-auth-code' ) ?? '';
		$encode     = md5( $code . '-' . $auth_id . '-' . $auth_key . '-' . $auth_time );
		// 先尝试从缓存中检查
		if ( $auth_cache !== $encode || $time - $auth_time > 7200 ) {
			// 不存在则重新获取
			$domain = preg_replace( '/^https?:\/\//', '', home_url() );
			$auth   = wp_remote_post( 'https://auth.haozi.net/api/v1/checkAuth', array(
				'body'        => json_encode( array(
					'product_id' => '1',
					'auth_id'    => $auth_id,
					'domain'     => $domain,
					'sign'       => md5( sprintf( 'id=%s&key=%s', strtoupper( $auth_id ), strtoupper( $auth_key ) ) ),
				) ),
				'sslverify'   => false,
				'headers'     => array( 'Content-Type' => 'application/json' ),
				'data_format' => 'body',
				'timeout'     => 60,
			) );
			if ( is_wp_error( $auth ) || wp_remote_retrieve_response_code( $auth ) !== 200 ) {
				return $this->error( '检查授权失败，如已购买请与作者联系' );
			}
			$auth = json_decode( wp_remote_retrieve_body( $auth ), true );
			if ( ! isset( $auth['code'] ) || (int) $auth['code'] !== 200 ) {
				return $this->error( '您的插件还未授权，请前往 https://hzbk.net/archives/61055.html 购买授权' );
			}
			// 将授权信息写入缓存
			$encode = md5( $code . '-' . $auth_id . '-' . $auth_key . '-' . $time );
			set_transient( 'wp-chatgpt-auth-code', $encode, 7200 );
			set_transient( 'wp-chatgpt-auth-time', $time, 7200 );
		}

		$options = $request->get_param( 'options' );

		if ( ! isset( $options['conversationId'] ) ) {
			return $this->error( '会话信息不存在' );
		}

		if ( empty( $request->get_param( 'prompt' ) ) ) {
			return $this->error( '请输入内容' );
		}
		// 判断敏感词
		if ( ! empty( $this->settings['sensitive-words'] ) ) {
			foreach ( $this->settings['sensitive-words'] as $word ) {
				if ( ! empty( $word ) && str_contains( $request->get_param( 'prompt' ), $word['sensitive-word'] ?? null ) ) {
					// 判断敏感词是否大于3个字符，如果大于3个字符则只显示第一个和最后一个字符
					if ( mb_strlen( $word['sensitive-word'] ) > 3 ) {
						$word['sensitive-word'] = mb_substr( $word['sensitive-word'], 0, 1 ) . str_repeat( '*', mb_strlen( $word['sensitive-word'] ) - 2 ) . mb_substr( $word['sensitive-word'], - 1 );
					}

					return $this->error( '您输入的内容包含敏感词：' . $word['sensitive-word'] );
				}
			}
		}
		// 百度云内容审核
		if ( ! empty( $this->settings['baidu-censoring'] ) && $this->settings['baidu-censoring'] ) {
			// 用ak/sk换取token
			$token = wp_remote_post( 'https://aip.baidubce.com/oauth/2.0/token', array(
				'body'        => array(
					'grant_type'    => 'client_credentials',
					'client_id'     => $this->settings['baidu-censoring-api-key'] ?? null,
					'client_secret' => $this->settings['baidu-censoring-secret-key'] ?? null,
				),
				'sslverify'   => false,
				'headers'     => array( 'Content-Type' => 'application/json' ),
				'data_format' => 'body',
				'timeout'     => 60,
			) );
			if ( is_wp_error( $token ) || wp_remote_retrieve_response_code( $token ) !== 200 ) {
				return $this->error( '初始化内容审核请求失败，请稍后再试' );
			}
			$token = json_decode( wp_remote_retrieve_body( $token ), true );
			if ( ! isset( $token['access_token'] ) ) {
				return $this->error( '初始化内容审核鉴权失败，请检查配置' );
			}
			$censor = wp_remote_post( 'https://aip.baidubce.com/rest/2.0/solution/v1/text_censor/v2/user_defined?access_token=' . $token['access_token'], array(
				'body'      => array(
					'text' => $request->get_param( 'prompt' ),
				),
				'sslverify' => false,
				'headers'   => array( 'Content-Type' => 'application/x-www-form-urlencoded' ),
				'timeout'   => 60,
			) );
			if ( is_wp_error( $censor ) || wp_remote_retrieve_response_code( $censor ) !== 200 ) {
				return $this->error( '内容审核失败，请稍后再试' );
			}
			$censor = json_decode( wp_remote_retrieve_body( $censor ), true );
			if ( ! isset( $censor['conclusionType'] ) || (int) $censor['conclusionType'] !== 1 ) {
				$msg = '';
				foreach ( $censor['data'] as $item ) {
					$msg .= $item['msg'] . '、';
				}
				$msg = rtrim( $msg, '、' );

				if ( empty( $msg ) ) {
					$msg = '审核返回异常';
					error_log( $censor );
				}

				return $this->error( '您输入的内容：' . $msg );
			}
		}

		// 尝试获取会话评论
		$conversation = get_post( $options['conversationId'] );
		if ( ! $conversation ) {
			return $this->error( '会话不存在' );
		}

		// 检查是否是当前用户的会话
		if ( $conversation->post_author != get_current_user_id() ) {
			return $this->error( '无权访问' );
		}

		// 判断额度是否足够
		// 获取额度
		if ( ! empty( $this->settings['pay-switch'] ) && $this->settings['pay-switch'] && get_current_user_id() != $this->settings['custom-conversation-user'] ?? null ) {
			$quota = get_user_meta( get_current_user_id(), 'wp-chatgpt-quota', true );
			if ( empty( $quota ) ) {
				$quota = 0;
			}
			if ( $quota <= 0 ) {
				return $this->error( '额度不足，请充值后再发送' );
			}
		}

		// 获取评论
		$comments = get_comments( [
			'post_id' => $conversation->ID,
		] );

		/**
		 * 检查是否符合要求
		 * 1. 评论为空，且有 parentMessageId 且 parentMessageId 不为 0 （首条消息 parentMessageId 为 0）
		 * 2. 评论不为空，且有 regenerate 但是没有 parentMessageId
		 */
		if ( empty( $comments ) && isset( $options['parentMessageId'] ) && (int) $options['parentMessageId'] != 0 ) {
			return $this->error( '消息错误，请刷新页面，若仍报错请删除重新开启会话' );
		}
		if ( ! empty( $comments ) && ! empty( $options['regenerate'] ) && empty( $options['parentMessageId'] ) ) {
			return $this->error( '消息错误，请刷新页面，若仍报错请删除重新开启会话' );
		}
		$options['parentMessageId'] = $options['parentMessageId'] ?? 0;

		// 倒序循环评论作为聊天记录
		$text_count = 0;
		$count      = count( $comments );
		// 为首次聊天自动设置标题
		if ( $count == 0 ) {
			wp_update_post( [
				'ID'         => $conversation->ID,
				'post_title' => $request->get_param( 'prompt' ),
			] );
		}

		// 取出最后一条消息作为父消息
		// 如果有 regenerate 则使用 parentMessageId 作为父消息
		$parent_message_id = $comments[0]->comment_ID ?? $options['parentMessageId'];
		if ( ! empty( $options['regenerate'] ) ) {
			$parent_message_id = $options['parentMessageId'] ?? 0;
		}

		// 随机选择账号
		$gpt_type = $this->settings['gpt-type'] ?? 'chatgpt-3.5';
		// 尝试在 openai-accounts中 搜索 account-select 为 $gpt_type 的账号
		$accounts = array_filter( $this->settings['openai-accounts'] ?? [], function ( $item ) use ( $gpt_type ) {
			return $item['account-select'] == $gpt_type;
		} );
		// 重置索引
		$accounts      = array_values( $accounts );
		$account_count = count( $accounts ?? [] );
		if ( $account_count == 0 ) {
			return $this->error( '请先配置 AI 账号' );
		}
		$index   = rand( 1, $account_count );
		$account = $accounts[ $index - 1 ];

		if ( $gpt_type == 'chatgpt-3.5' ) {
			$chat_gpt      = new V2( $account['account-secret-key'] ?? null, $this->settings['openai-api-server'] ?? null, $account['chatgpt-api-model'] ?? 'gpt-3.5-turbo' );
			$system_prompt = empty( $this->settings['chatgpt-system-prompt'] ) ? '你是 ChatGPT，一个由 OpenAI 训练的大型语言模型，模型是 GPT-3.5，当前时间是 {datetime}。尽可能简洁地回答。' : $this->settings['chatgpt-system-prompt'];
			// 尝试替换系统提示中的变量
			$system_prompt = str_replace( '{datetime}', date( 'Y-m-d H:i:s' ), $system_prompt );

			$chat_gpt->addMessage( $system_prompt, 'system' );

			for ( $i = $count - 1; $i >= 0; $i -- ) {
				$comment = $comments[ $i ];
				// 判断一下$comment->comment_ID是否大于等于$options['parentMessageId'] + 1
				// 这是为了在重载时，不添加新的评论
				// +1 是因为 parentMessageId 是最后一条问题的ID，而不是最后一条回答的ID
				if ( isset( $options['parentMessageId'] ) && $comment->comment_ID >= $options['parentMessageId'] + 1 ) {
					// 如果大于等于，跳过
					continue;
				}
				// 取评论的meta
				$inversion = get_comment_meta( $comment->comment_ID, 'inversion', true );// yes: 用户提问，no: ChatGPT 回答
				if ( ! empty( $comment->comment_content ) ) {
					$chat_gpt->addMessage( $comment->comment_content, $inversion === 'yes' ? 'user' : 'assistant' );
					// 统计content的字数
					$text_count += mb_strlen( $comment->comment_content );
				}
				if ( $text_count > 1000 ) {
					if (isset($account['chatgpt-api-model']) && $account['chatgpt-api-model'] == 'gpt-3.5-turbo') {
						break;
					}
				}
				if ( $text_count > 2000 ) {
					if (isset($account['chatgpt-api-model']) && $account['chatgpt-api-model'] == 'gpt-4') {
						break;
					}
				}
				if ( $text_count > 4000 ) {
					if (isset($account['chatgpt-api-model']) && $account['chatgpt-api-model'] == 'gpt-3.5-turbo-16k') {
						break;
					}
				}
			}

			try {
				if ( $this->settings['stream-output'] ?? true ) {
					$answers = $chat_gpt->ask( $request->get_param( 'prompt' ), null, true );
				} else {
					$answers = $chat_gpt->ask( $request->get_param( 'prompt' ) );
				}

			} catch ( Exception $e ) {
				// 替换错误信息中的私有域名
				$error = $e->getMessage();
				if ( $this->settings['openai-api-server'] ?? false ) {
					$error = str_replace( $this->settings['openai-api-server'], '****.****.****', $error );
				}
				$error .= "\n";
				$error .= 'Account: ' . $account['account-note'] ?? '未知';

				return $this->error( $error );
			}
		} else if ( $gpt_type == 'chatgpt-web' ) {
			$chat_gpt = new V1( $this->settings['openai-api-server'] ?? null );
			// 尝试从会话元数据中读取 access_token 和 conversation_id
			// 从消息中读取 chatgpt_parent_id
			$access_token            = get_post_meta( $conversation->ID, 'openai_access_token', true );
			$chatgpt_conversation_id = get_post_meta( $conversation->ID, 'chatgpt_conversation_id', true );
			$chatgpt_parent_id       = ( ! empty( $options['parentMessageId'] ) && $options['parentMessageId'] != 0 ) ? get_comment_meta( $options['parentMessageId'], 'chatgpt_message_id', true ) : null;
			// 如果没有，就从账号中读取
			if ( empty( $access_token ) ) {
				$access_token = $account['account-access-token'] ?? null;
			}
			// 如果会话元数据为空，就将其重置为null
			if ( empty( $chatgpt_conversation_id ) || empty( $chatgpt_parent_id ) ) {
				$chatgpt_conversation_id = null;
				$chatgpt_parent_id       = null;
			}

			$chat_gpt->addAccount( $access_token, null, false, $account['chatgpt-web-model'] ?? 'text-davinci-002-render-sha' );

			try {
				if ( $this->settings['stream-output'] ?? true ) {
					$answers = $chat_gpt->ask( $request->get_param( 'prompt' ), $chatgpt_conversation_id, $chatgpt_parent_id, null, true );
				} else {
					$answers = $chat_gpt->ask( $request->get_param( 'prompt' ), $chatgpt_conversation_id, $chatgpt_parent_id );
				}

			} catch ( Exception $e ) {
				// 替换错误信息中的私有域名
				$error = $e->getMessage();
				if ( $this->settings['openai-api-server'] ?? false ) {
					$error = str_replace( $this->settings['openai-api-server'], '****.****.****', $error );
				}
				$error .= "\n";
				$error .= 'Account: ' . $account['account-note'] ?? '未知';

				return $this->error( $error );
			}

			// 保存access_token
			update_post_meta( $conversation->ID, 'openai_access_token', $access_token );
		} else {
			// 文心一言

			// 尝试从会话元数据中读取 文心一言账号cookie 和 session_id
			// 从消息中读取 chatgpt_parent_id
			$yiyan_cookie     = get_post_meta( $conversation->ID, 'yiyan_cookie', true );
			$yiyan_session_id = get_post_meta( $conversation->ID, 'yiyan_session_id', true );
			$yiyan_parent_id  = ( ! empty( $options['parentMessageId'] ) && $options['parentMessageId'] != 0 ) ? get_comment_meta( $options['parentMessageId'], 'yiyan_chat_id', true ) : null;
			// 如果没有，就从账号中读取
			if ( empty( $yiyan_cookie ) ) {
				$yiyan_cookie = $account['account-yiyan-cookie'] ?? null;
			}
			// 如果会话元数据为空，就将其重置为null
			if ( empty( $yiyan_session_id ) || empty( $yiyan_parent_id ) ) {
				$yiyan_session_id = null;
				$yiyan_parent_id  = null;
			}
			// 保存cookie
			update_post_meta( $conversation->ID, 'yiyan_cookie', $yiyan_cookie );

			$yiyan = new Yiyan( $yiyan_cookie );
			try {
				$answers = $yiyan->ask( $request->get_param( 'prompt' ), $yiyan_session_id, $yiyan_parent_id );
			} catch ( Exception|GuzzleException $e ) {
				$error = $e->getMessage();
				$error .= "\n";
				$error .= 'Account: ' . $account['account-note'] ?? '未知';

				return $this->error( $error );
			}
		}

		// 为重载，不保存提问
		if ( empty( $options['regenerate'] ) ) {
			global $current_user;
			$question_id = wp_insert_comment( [
				'comment_post_ID' => $conversation->ID,
				'comment_author'  => $current_user->display_name,
				'comment_content' => $request->get_param( 'prompt' ),
				'user_id'         => $current_user->ID,
				'comment_meta'    => [
					'parent_message_id' => $parent_message_id,
					'inversion'         => 'yes',
					'prompt'            => $request->get_param( 'prompt' ),
				],
			] );
		} else {
			$question_id = $options['parentMessageId'];
		}

		// 保存回答
		if ( empty( $options['regenerate'] ) ) {
			global $current_user;
			$answer_id = wp_insert_comment( [
				'comment_post_ID' => $conversation->ID,
				'comment_author'  => 'AI',
				'comment_content' => '',
				'user_id'         => $current_user->ID,
				'comment_meta'    => [
					'parent_message_id' => $question_id,
					'inversion'         => 'no',
					'prompt'            => $request->get_param( 'prompt' ),
				],
			] );
		} else {
			wp_update_comment( [
				'comment_ID'      => $options['regenerate'],
				'comment_content' => '',
			] );
			$answer_id = $options['regenerate'];
		}

		// 判断流式输出
		header( 'Content-type: application/octet-stream' );
		header( 'Cache-Control: no-cache' );
		header( 'X-Accel-Buffering: no' );
		header( 'Access-Control-Allow-Origin: *' );
		ob_end_clean();
		if ( $this->settings['stream-output'] ?? true ) {
			$temp               = '';
			$message_id         = '';
			$first              = true;
			$temp_conversion_id = null;
			$now                = '';
			if ( $gpt_type == 'yiyan' ) {
				try {
					foreach ( $answers as $answer ) {

						if ( ! $first ) {
							echo "\n";
						}
						$first = false;
						echo json_encode( [
							"role"            => "assistant",
							"id"              => $answer_id,
							'conversationId'  => $conversation->ID,
							"parentMessageId" => $question_id,
							"text"            => $answer['answer'],
							"delta"           => $answer['now'],
						], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_LINE_TERMINATORS );
						echo "\n";
						echo json_encode( [
							"role"            => "assistant",
							"id"              => $answer_id,
							'conversationId'  => $conversation->ID,
							"parentMessageId" => $question_id,
							"text"            => $answer['answer'],
							"delta"           => $answer['now'],
						], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_LINE_TERMINATORS );
						$temp = $answer['answer'];
						// 判断敏感词
						if ( ! empty( $this->settings['sensitive-words'] ) ) {
							foreach ( $this->settings['sensitive-words'] as $word ) {
								if ( ! empty( $word ) && str_contains( $temp, $word['sensitive-word'] ?? null ) ) {
									return $this->error( '返回的内容包含敏感词' );
								}
							}
						}
						ob_flush();
						flush();
						$message_id         = $answer['chat_id'];
						$temp_conversion_id = $answer['session_id'];

					}
				} catch ( Exception $e ) {
					$error = $e->getMessage();
					$error .= "\n";
					$error .= 'Account: ' . $account['account-note'] ?? '未知';

					return $this->error( $error );
				}
			} else {
				try {
					foreach ( $answers as $answer ) {
						if ( ! $first ) {
							echo "\n";
						}
						$first = false;
						if ( $gpt_type == 'chatgpt-web' ) {
							$temp   = $answer['answer'];
							$single = str_replace( $now, '', $temp );
							$now    = $temp;
						} else {
							$temp .= $answer['answer'];
						}

						echo json_encode( [
							"role"            => "assistant",
							"id"              => $answer_id,
							'conversationId'  => $conversation->ID,
							"parentMessageId" => $question_id,
							"text"            => $temp,
							"delta"           => $gpt_type == 'chatgpt-web' ? $single : $answer['answer'],
						], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_LINE_TERMINATORS );
						// 判断敏感词
						if ( ! empty( $this->settings['sensitive-words'] ) ) {
							foreach ( $this->settings['sensitive-words'] as $word ) {
								if ( ! empty( $word ) && str_contains( $temp, $word['sensitive-word'] ?? null ) ) {
									return $this->error( '返回的内容包含敏感词' );
								}
							}
						}
						if ( $gpt_type == 'chatgpt-web' ) {
							$temp_conversion_id = $answer['conversation_id'];
						}
						$message_id = $answer['id'];
						ob_flush();
						flush();
					}
				} catch ( Exception $e ) {
					$error = $e->getMessage();
					$error .= "\n";
					$error .= 'Account: ' . $account['account-note'] ?? '未知';

					return $this->error( $error );
				}
			}

			// 更新回答meta
			update_comment_meta( $answer_id, 'message_id', $message_id );
			wp_update_comment( [
				'comment_ID'      => $answer_id,
				'comment_content' => $temp,
			] );

			// 如果是网页版，更新会话id和父消息id
			if ( $gpt_type == 'chatgpt-web' ) {
				update_post_meta( $conversation->ID, 'chatgpt_conversation_id', $temp_conversion_id );
				update_comment_meta( $question_id, 'chatgpt_message_id', $message_id );
			}
			// 文心一言也是这个操作
			if ( $gpt_type == 'yiyan' ) {
				update_post_meta( $conversation->ID, 'yiyan_session_id', $temp_conversion_id );
				update_comment_meta( $question_id, 'yiyan_chat_id', $message_id );
			}

			// 更新额度
			if ( ! empty( $this->settings['pay-switch'] ) && $this->settings['pay-switch'] && get_current_user_id() != $this->settings['custom-conversation-user'] ?? null ) {
				$quota = get_user_meta( get_current_user_id(), 'wp-chatgpt-quota', true );
				$quota --;
				update_user_meta( get_current_user_id(), 'wp-chatgpt-quota', $quota );
			}

		} else {

			$temp_conversion_id = '';
			$message_id         = '';
			$answer_text        = '';

			if ( $gpt_type == 'yiyan' ) {
				try {
					foreach ( $answers as $answer ) {
						$message_id         = $answer['chat_id'];
						$temp_conversion_id = $answer['session_id'];
						$answer_text        = $answer['answer'];
					}
				} catch ( Exception $e ) {
					$error = $e->getMessage();
					$error .= "\n";
					$error .= 'Account: ' . $account['account-note'] ?? '未知';

					return $this->error( $error );
				}

				// 判断敏感词
				if ( ! empty( $this->settings['sensitive-words'] ) ) {
					foreach ( $this->settings['sensitive-words'] as $word ) {
						if ( ! empty( $word ) && str_contains( $answer_text, $word['sensitive-word'] ?? null ) ) {
							return $this->error( '返回的内容包含敏感词' );
						}
					}
				}
				echo json_encode( [
					"role"            => "assistant",
					"id"              => $answer_id,
					'conversationId'  => $conversation->ID,
					"parentMessageId" => $question_id,
					"text"            => $answer_text,
					"delta"           => $answer_text,
				], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_LINE_TERMINATORS );
				echo PHP_EOL;
				echo json_encode( [
					"role"            => "assistant",
					"id"              => $answer_id,
					'conversationId'  => $conversation->ID,
					"parentMessageId" => $question_id,
					"text"            => $answer_text,
					"delta"           => $answer_text,
				], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_LINE_TERMINATORS );
			} else {
				try {
					foreach ( $answers as $answer ) {
						// 判断敏感词
						if ( ! empty( $this->settings['sensitive-words'] ) ) {
							foreach ( $this->settings['sensitive-words'] as $word ) {
								if ( ! empty( $word ) && str_contains( $answer['answer'], $word['sensitive-word'] ?? null ) ) {
									return $this->error( '返回的内容包含敏感词' );
								}
							}
						}
						echo json_encode( [
							"role"            => "assistant",
							"id"              => $answer_id,
							'conversationId'  => $conversation->ID,
							"parentMessageId" => $question_id,
							"text"            => $answer['answer'],
							"delta"           => $answer['answer'],
						], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_LINE_TERMINATORS );
						echo PHP_EOL;
						echo json_encode( [
							"role"            => "assistant",
							"id"              => $answer_id,
							'conversationId'  => $conversation->ID,
							"parentMessageId" => $question_id,
							"text"            => $answer['answer'],
							"delta"           => $answer['answer'],
						], JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_LINE_TERMINATORS );

						if ( $gpt_type == 'chatgpt-web' ) {
							$temp_conversion_id = $answer['conversation_id'];
						}
						$message_id = $answer['id'];
						ob_flush();
						flush();
					}
				} catch ( Exception $e ) {
					$error = $e->getMessage();
					$error .= "\n";
					$error .= 'Account: ' . $account['account-note'] ?? '未知';

					return $this->error( $error );
				}
			}

			// 更新回答meta
			update_comment_meta( $answer_id, 'message_id', $message_id );
			wp_update_comment( [
				'comment_ID'      => $answer_id,
				'comment_content' => $message_id,
			] );

			// 如果是网页版，更新会话id和父消息id
			if ( $gpt_type == 'chatgpt-web' ) {
				update_post_meta( $conversation->ID, 'chatgpt_conversation_id', $temp_conversion_id );
				update_comment_meta( $question_id, 'chatgpt_message_id', $message_id );
			}
			// 文心一言也是这个操作
			if ( $gpt_type == 'yiyan' ) {
				update_post_meta( $conversation->ID, 'yiyan_session_id', $temp_conversion_id );
				update_comment_meta( $question_id, 'yiyan_chat_id', $message_id );
			}

			// 更新额度
			if ( ! empty( $this->settings['pay-switch'] ) && $this->settings['pay-switch'] ) {
				$quota = get_user_meta( get_current_user_id(), 'wp-chatgpt-quota', true );
				$quota --;
				update_user_meta( get_current_user_id(), 'wp-chatgpt-quota', $quota );
			}

			ob_flush();
			flush();
		}

		exit;
	}

	/**
	 * 删除消息
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_REST_Response
	 */
	public function delete_message( WP_REST_Request $request ): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 获取参数
		$conversation_id = $request->get_param( 'uuid' );
		$message_id      = $request->get_param( 'message' );

		// 获取会话文章
		$post = get_post( $conversation_id );
		if ( ! $post ) {
			return $this->error( '会话不存在' );
		}

		// 检查是否是当前用户的会话
		if ( (int) $post->post_author != get_current_user_id() ) {
			return $this->error( '无权删除' );
		}

		// 获取评论
		$comment = get_comment( $message_id );
		if ( ! $comment ) {
			return $this->error( '消息不存在' );
		}

		// 检查是否是当前用户的评论
		if ( (int) $comment->user_id != get_current_user_id() ) {
			return $this->error( '无权删除' );
		}

		// 判断是否是使用网页版
		if ( $this->settings['gpt-type'] == 'chatgpt-web' ) {
			return $this->error( '网页版后端消息无法删除' );
		}

		// 判断文心一言
		if ( $this->settings['gpt-type'] == 'yiyan' ) {
			return $this->error( '文心一言后端消息无法删除' );
		}

		// 删除评论
		if ( $this->settings['message-trash'] ?? false ) {
			wp_trash_comment( $comment->comment_ID );
		} else {
			wp_delete_comment( $comment->comment_ID, true );
		}

		// 返回数据
		return $this->success( '删除成功' );
	}

	/**
	 * 获取会话列表
	 *
	 * @return WP_REST_Response
	 */
	public function conversation_list(): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 移除过滤器
		kses_remove_filters();

		// 获取当前用户的会话文章
		$posts = get_posts( [
			'post_type'      => 'cg_conversations',
			'posts_per_page' => - 1,
			'author'         => get_current_user_id(),
		] );

		// 如果没有会话，新建一个
		if ( empty( $posts ) ) {
			wp_insert_post( [
				'post_title'  => "New Chat",
				'post_type'   => 'cg_conversations',
				'post_status' => 'publish',
				'post_author' => get_current_user_id(),
			] );
			$posts = get_posts( [
				'post_type'      => 'cg_conversations',
				'posts_per_page' => - 1,
				'author'         => get_current_user_id(),
			] );
		}

		// 取每个会话的所有评论
		$messages = [];
		foreach ( $posts as $post ) {
			$comments = get_comments( [
				'post_id' => $post->ID,
				'order'   => 'ASC',
			] );
			foreach ( $comments as $comment ) {
				$parent_message_id = get_comment_meta( $comment->comment_ID, 'parent_message_id', true );
				$inversion         = get_comment_meta( $comment->comment_ID, 'inversion', true );
				$prompt            = get_comment_meta( $comment->comment_ID, 'prompt', true );

				$messages[ $post->ID ][] = [
					'conversationOptions' => [
						'conversationId'  => $comment->comment_post_ID,
						'parentMessageId' => $parent_message_id,
					],
					'id'                  => $comment->comment_ID,
					'dateTime'            => $comment->comment_date,
					'error'               => false,
					'inversion'           => $inversion === 'yes',
					'text'                => $comment->comment_content,
					'requestOptions'      => [
						'options' => [
							'conversationId'  => $comment->comment_post_ID,
							'parentMessageId' => $parent_message_id,
						],
						'prompt'  => $prompt, // 用户提问的内容
					],
				];
			}
		}

		// 构建返回数据
		$data = [];
		foreach ( $posts as $post ) {
			// 获取文章的 is_edit 字段
			$is_edit           = get_post_meta( $post->ID, 'is_edit', true );
			$data['history'][] = [
				'uuid'   => $post->ID,
				'title'  => $post->post_title,
				'isEdit' => ! empty( $is_edit ),
			];
			$data['chat'][]    = [
				'uuid' => $post->ID,
				'data' => $messages[ $post->ID ] ?? [],
			];
		}

		// 返回数据
		return $this->success( '获取成功', $data );
	}

	/**
	 * 新会话
	 *
	 * @return WP_REST_Response
	 */
	public function new_conversation(): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 移除过滤器
		kses_remove_filters();

		// 创建文章
		$post_id = wp_insert_post( [
			'post_title'  => "New Chat",
			'post_type'   => 'cg_conversations',
			'post_status' => 'publish',
			'post_author' => get_current_user_id(),
		] );

		// 返回数据
		return $this->success( '创建成功', [
			'uuid' => $post_id,
		] );
	}

	/**
	 * 更新会话
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_REST_Response
	 */
	public function update_conversation( WP_REST_Request $request ): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 获取参数
		$conversation_id = $request->get_param( 'uuid' );
		$title           = $request->get_param( 'title' );

		// 获取会话文章
		$post = get_post( $conversation_id );
		if ( ! $post ) {
			return $this->error( '会话不存在' );
		}

		// 检查是否是当前用户的会话
		if ( $post->post_author != get_current_user_id() ) {
			return $this->error( '无权修改' );
		}

		// 更新文章
		wp_update_post( [
			'ID'         => $conversation_id,
			'post_title' => $title,
		] );

		// 返回数据
		return $this->success( '修改成功' );
	}

	/**
	 * 删除会话
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_REST_Response
	 */
	public function delete_conversation( WP_REST_Request $request ): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 获取参数
		$conversation_id = $request->get_param( 'uuid' );

		// 获取会话文章
		$post = get_post( $conversation_id );
		if ( ! $post ) {
			return $this->error( '会话不存在' );
		}

		// 检查是否是当前用户的会话
		if ( $post->post_author != get_current_user_id() ) {
			return $this->error( '无权删除' );
		}

		// 删除文章
		if ( $this->settings['conversation-trash'] ?? false ) {
			wp_trash_post( $post->ID );
		} else {
			wp_delete_post( $post->ID, true );
		}

		// 返回数据
		return $this->success( '删除成功' );
	}

	/**
	 * 清空会话消息
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_REST_Response
	 */
	public function clear_conversation( WP_REST_Request $request ): WP_REST_Response {
		// 检查是否登录
		if ( ! is_user_logged_in() ) {
			return $this->error( $this->no_login_notice );
		}

		// 获取参数
		$conversation_id = $request->get_param( 'uuid' );

		// 获取会话文章
		$post = get_post( $conversation_id );
		if ( ! $post ) {
			return $this->error( '会话不存在' );
		}

		// 检查是否是当前用户的会话
		if ( $post->post_author != get_current_user_id() ) {
			return $this->error( '无权删除' );
		}

		// 判断是否是使用网页版
		if ( $this->settings['gpt-type'] == 'chatgpt-web' ) {
			return $this->error( '网页版后端消息无法清空' );
		}

		// 判断文心一言
		if ( $this->settings['gpt-type'] == 'yiyan' ) {
			return $this->error( '文心一言后端消息无法清空' );
		}

		// 获取评论
		$comments = get_comments( [
			'post_id' => $post->ID,
		] );

		// 删除评论
		foreach ( $comments as $comment ) {
			if ( $this->settings['message-trash'] ?? false ) {
				wp_trash_comment( $comment->comment_ID );
			} else {
				wp_delete_comment( $comment->comment_ID, true );
			}
		}

		// 返回数据
		return $this->success( '清空成功' );
	}

	public function success( $message = 'success', $data = null ): WP_REST_Response {
		return new WP_REST_Response( array( 'message' => $message, 'data' => $data, 'status' => 'Success' ) );
	}

	public function error( $message = 'error', $data = null ): WP_REST_Response {
		return new WP_REST_Response( array( 'message' => $message, 'data' => $data, 'status' => 'Failed' ) );
	}

}
